package root

import (
	"code.wuban.net.cn/service/pancakeswapper/config"
	"code.wuban.net.cn/service/pancakeswapper/core"
	"fmt"
	log "github.com/sirupsen/logrus"
	"github.com/spf13/cobra"
	"github.com/spf13/pflag"
	"github.com/spf13/viper"
	"os"
	"strings"
)

var (
	cfgFile string
	rootCmd = &cobra.Command{
		Use:   "core [command]",
		Short: "Swapper is a script for pancake.",
		PersistentPreRunE: func(cmd *cobra.Command, args []string) error {
			bindFlags(cmd)
			return config.InitConfig(cfgFile)
		},
		Run: runCommand,
	}
)

func Execute() {
	if err := rootCmd.Execute(); err != nil {
		_, _ = fmt.Fprintln(os.Stderr, err)

		os.Exit(1)
	}
}

func init() {
	rootCmd.PersistentFlags().StringVar(&cfgFile, "config", "", "config file (default is ./config.yaml)")
	// add param config
	rootCmd.PersistentFlags().Int64("param.volume", config.DefaultConfig.Param.Volume, "volume")
	rootCmd.PersistentFlags().Int("param.count", config.DefaultConfig.Param.Count, "count")
	rootCmd.PersistentFlags().Int64("param.gas_price", config.DefaultConfig.Param.GasPrice, "gas price")
	rootCmd.PersistentFlags().Int64("param.slippage", config.DefaultConfig.Param.Slippage, "slippage")
	rootCmd.PersistentFlags().Int64("param.deadline", config.DefaultConfig.Param.Deadline, "deadline")

	// users config
	rootCmd.PersistentFlags().String("users", config.DefaultConfig.Users, "users file")

	rootCmd.PersistentFlags().String("log.level", config.DefaultConfig.Log.Level, "Log level")
}

func bindFlags(cmd *cobra.Command) {
	cmd.Flags().VisitAll(func(f *pflag.Flag) {
		if strings.Contains(f.Name, ".") {
			envVar := strings.ToUpper(strings.ReplaceAll(f.Name, ".", "_"))
			viper.BindEnv(f.Name, envVar)
		}

		viper.BindPFlag(f.Name, f)
	})
}

func runCommand(cmd *cobra.Command, _ []string) {
	cfg := config.Global
	setlog(cfg.Log.Level)
	log.WithField("config", cfg).Info("load config success")

	server, err := core.NewSwapper(cfg)
	if err != nil {
		log.Fatal(err)
	}
	server.Run()

	fmt.Println("app exit, bye bye !!!")
	return
}

func setlog(level string) {
	log.SetFormatter(&log.TextFormatter{
		DisableColors: true,
		FullTimestamp: true,
	})
	switch level {
	case "debug":
		log.SetLevel(log.DebugLevel)
	case "info":
		log.SetLevel(log.InfoLevel)
	case "warn":
		log.SetLevel(log.WarnLevel)
	case "error":
		log.SetLevel(log.ErrorLevel)
	default:
		log.SetLevel(log.DebugLevel)
	}
}
